// Multiple Terrain Creator Mod
// Research done by Sentinal and Raven, miscellaneous testing done by Powio
//---------------------------------------------------------------------
// FOR ADVANCED SCRIPTERS ONLY!!! (NOT FOR BEGINNERS)
//---------------------------------------------------------------------
// Installation:
// Place this file in your Starsiege main directory.
// This file will temporarily alter how Starsiege creates new missions,
// so it is recommended that you only use this file when you want to
// create multiple terrains. Otherwise, you should remove it when not in use.
//---------------------------------------------------------------------
// Step 1:
// Change the settings below for the EXISTING terrain you want to use. The example below lists CA1 from
// the cybrid campaign folder. If you want to use a multiplayer terrain, leave the $folder variable blank.
// NOTE: The altitudes at which you can use terrains (z-coordinates) have an indeterminate range. Setting
// the terrains' altitudes too far from zero (in the negative or positive directions) can sometimes cause
// strange things to start happening, such as no gravity, strange bouncing, the inability to turn your vehicle,
// and sometimes random places in the map where your vehicle might explode. So stay conservative when possible.
// But you are welcome to try and prove me wrong :P
//---------------------------------------------------------------------
// Step 2:
// Start the mission editor and select a map that uses the EXACT same terrain as the one you wish to use. Also
// be sure to choose a map that is also the same time of day as you wish to use. It is easiest to just open
// the same map you wish to use anyway (saves time and effort). If you want to use a campaign terrain, you may need
// to temporarily copy its terrain and mission into the multiplayer folder for this step.
//---------------------------------------------------------------------
// Step 3: Once the map has been loaded in the mission editor, click "New". 
// Then type a name for your mission (any name will do). Then select the same world type and time of day to
// be the same as the settings you entered in this file. The type of terrain (i.e. flat, rolling hills, rough, etc)
// does NOT matter, any setting will do, because we are not actually creating a new terrain.
// NOTE: This file changes what the "New" button does, so it will NOT create a new terrain. It will
// instead use the existing terrain you selected in the settings below.
//---------------------------------------------------------------------
// Step 4: Next, click "Create" and after a few seconds or so, you will be taken back to the main menu screen with
// an error message displayed saying that Starsiege cannot find the DTF file associated with the terrain (blah blah).
// Don't worry about it, that is supposed to happen. Now exit Starsiege and open your Starsiege main directory.
// You should find the .mis file you created (by the name you selected) in this folder. Note that Starsiege didn't
// create a .ted.vol like usual, it was modified so it wouldn't. Next, copy the .mis file into your multiplayer
// folder, and restart the mission editor.
//---------------------------------------------------------------------
// Step 5:
// This time, select YOUR mission which you just created. If the map doesn't load the first time (or crashes),
// just restart the editor and try again. Most likely, it will load on the second try. Once the map has loaded,
// make sure your terrains are positioned how you wanted them. If they are not in the correct place, you'll
// need to repeat the entire process all over again. However, if they ARE how you want them, you can then proceed
// to edit your map just like any other. However, due to the fact that there are several terrains in your
// map, moving the mission editor's camera around will be very difficult, and thus your map will be very hard
// to edit. Most of your editing will be done from a long distance away from your objects, and positioning objects
// correctly can become very tedious. I find that after I position an object, it is very helpful to look at it
// in-game, and correct its position accordingly (HercBoost helps a lot). And be sure to save often, the editor can
// get a little unstable at times due to several terrains being loaded simultaneously. And whatever you do,
// DO NOT change the terrains in any way while in the editor or you WILL crash! 
//---------------------------------------------------------------------
// Happy editing! I know you will get very frustrated the first few times you try this. :P
//---------------------------------------------------------------------

//Multiple Terrain Creator Settings

$terrainName = "CA1";      //Terrain you want to use (needs to be the same as the map and settings you select)
$folder = "campaign\\Cybrid (Advanced)\\";    //Terrain's folder; Leave blank for multiplayer folder

$visibleDistance = 3000;   //Visible distance (note that only the floor terrain seems to follow these settings)
$hazeDistance = 2000;      //Haze distance (note that only the floor terrain seems to follow these settings)

//Floor
$terrain1Xoffset = -12288; //Recommended setting for x-coordinate
$terrain1Yoffset = -12288; //Recommended setting for y-coordinate
$terrain1Zoffset = -500;   //The z-coordinates will need to be tweaked uniquely for each terrain by trial & error
$terrain1Xrotation = 0;    //Angle at which terrain 1 will be rotated about the x-axis (east-west)
$terrain1Yrotation = 0;    //Angle at which terrain 1 will be rotated about the y-axis (north-south)
$terrain1Zrotation = 0;    //Angle at which terrain 1 will be rotated about the z-axis (up-down)

//Ceiling
$terrain2Xoffset = 12288;  //Recommended setting for x-coordinate
$terrain2Yoffset = -12288; //Recommended setting for y-coordinate
$terrain2Zoffset = 0;      //The z-coordinates will need to be tweaked uniquely for each terrain by trial & error
$terrain2Xrotation = 180;  //Angle at which terrain 2 will be rotated about the x-axis (east-west)
$terrain2Yrotation = 0;    //Angle at which terrain 2 will be rotated about the y-axis (north-south)
$terrain2Zrotation = 180;  //Angle at which terrain 2 will be rotated about the z-axis (up-down)

//Sub-floor 
//This terrain is necessary to fix a bug with vehicle movements, particularly with tanks (see notes on sub-floor below)
//The subfloor should be directly below the floor (hence the x and y coordinates are the same as terrain1)
//The subfloor's z-coordinate should be offset about 300-400 meters below the floor (terrain1)
$terrain3Zoffset = -900;   //The z-coordinates will need to be tweaked uniquely for each terrain by trial & error

// Notes on Sub-floor
//---------------------------------------------------------------------
// The third "sub-floor" terrain is necessary to fix a bug with vehicle movements, particularly with tanks.
// As you probably already know, tanks hug the terrain and tilt with it (unlike hercs).
// Essentially, if only 2 terrains were used (floor and ceiling), then the tank would tend to tilt with the ceiling
// terrain (instead of the floor). Obviously, this could cause a lot of problems with the tank's ability to move.
// Therefore, the solution we found was to use a third terrain placed directly below the floor (called the sub-floor)
// to counteract the effect which the ceiling has on the tanks. It seems to work very effectively, and it even fixes
// some minor problems with herc movement as well. So basically, this is why we use the third terrain.
//---------------------------------------------------------------------

// Other Comments
//---------------------------------------------------------------------
// BTW, if you really know what you are doing, you can modify this to load as many terrains as Starsiege will allow.
// For example, a 6-terrain "cube" based on the Cold_Titan_Night terrain has already been successfully tested.
// Though lag issues would have to be taken into consideration :P
//---------------------------------------------------------------------

function NewMission::setValues()
{
   // do 'WorldType'
   Popup::clear(WorldType);
   for(%i = 0; $worldTypes[%i, type] != ""; %i++)
      Popup::addLine(WorldType, $worldTypes[%i, type], %i);
   Popup::setSelected(WorldType, $NM::worldIndex);
   
   // do 'skytype'
   Popup::clear(SkyType);
   %wi = $NM::worldIndex;
   for(%i = 0; $worldTypes[%wi, skyname, %i] != ""; %i++)
      Popup::addLine(SkyType, $worldTypes[%wi, skyname, %i], %i);
   Popup::setSelected(SkyType, $NM::skyIndex);
   
   // do 'terraintype'
   Popup::clear(TerrainType);
   for(%i = 0; $terrainTypes[%i, type] != ""; %i++)
      Popup::addLine(TerrainType, $terrainTypes[%i, description], %i);
   Popup::setSelected(TerrainType, $NM::terrainIndex);
}

//---------------------------------------------------------------------
// callbacks

function WorldType::onSelect(%string, %level)
{
   if($NM::worldIndex != %level)
   {
      $NM::worldIndex = %level;
      $NM::skyIndex = 0;
   }
   
   NewMission::setValues();
}

function SkyType::onSelect(%string, %level)
{
   $NM::skyIndex = %level;
   NewMission::setValues();
}

function TerrainType::onSelect(%string, %level)
{
   $NM::terrainIndex = %level;
   NewMission::setValues();
}

//-----------------------------------------------------------------------

function NewMission::create()
{
   %si          = $NM::skyIndex;
   %wi          = $NM::worldIndex;
   %ti          = $NM::terrainIndex;
   %missionName = convertSpaces($NM::missionName);
   %palette     = $worldTypes[%wi, palette, %si];
   %world       = $worldTypes[%wi, type];
   %simVol      = strcat( %world, ".sim.vol" );
   %dmlName     = strcat( %world, ".terrain.dml" );
   %gridName    = strcat( %world, ".grid.dat" );
   %rulesName   = strcat( %missionName, ".rules.dat" );
   %terrainDTF  = strcat( %missionName, ".ted.dtf" );
   %tedVol      = strcat( $Ted::currPath, %missionName, ".ted.vol" );
   %scriptName    = strcat( $Ted::currPath, %missionName, ".cs" );
   $server::MissionFileName = strcat( $Ted::currPath, %missionName @ ".mis" );
   $server::Mission           = %missionName;

   UnFocusCamera( $ME::camera );
   unfocus(TedObject);
   unfocus(MissionEditor);
   GuiLoadContentCtrl("simCanvas", "play.gui");
   MissionObjectList::ClearDisplayGroups();
   focusServer();
   deleteObject("MissionGroup");
   deleteObject("MissionCleanup");
   purgeDebris();
   resetPlayerManager();
   setGameInfo("");
   flushPilots();
   flushDustManager();
   purgeResources();
   focusClient();
   purgeResources();
   
   focusServer();

   deleteFunctions("*::vehicle::*");
   deleteFunctions("vehicle::*");
   deleteFunctions("*::structure::*");
   deleteFunctions("structure::*");
   deleteFunctions("*::turret::*");
   deleteFunctions("turret::*");
   deleteFunctions("*::trigger::*");
   deleteFunctions("trigger::*");
   deleteFunctions("onSPClientInit");

   exec( "missionStdLib.cs" );
   $server::HudMapViewOffsetX = 0;
   $server::HudMapViewOffsetY = 0;
   setHudMapViewOffset($server::HudMapViewOffsetX, $server::HudMapViewOffsetY);

   // --------------- default objects ------------------

   newObject( "MissionGroup", SimGroup );
   newObject( "MissionCleanup", SimGroup );
   newObject( "Volumes", SimGroup );
   newObject( "World", SimGroup );
   newObject( "DropPoints", SimGroup );
   addToSet( "MissionGroup", "Volumes", "World", "DropPoints" );

   newObject( terrainVol, SimVolume, %simVol );
   addToSet( "MissionGroup\\Volumes", "terrainVol" );

   newObject( "terrainPal", "ESPalette", %palette );
   newObject( "sky", "SimSky", 0,0,0, $worldTypes[%wi,skydml,%si], 0, $worldTypes[%wi,panels,%si] );
   newObject( "planet", "SimPlanet", $worldTypes[%wi, planet, %si], 0, 30, t, t, 1, 1, 1, 0, 0, 0 );

   // ---------------- create terrain ------------------
   
   // For multiple terrains, this section needs to be commented out
   //newObject( "terrain", "SimTerrain", "Create", %terrainDTF, 12, 5, 64, 0, 2, -12288, -12288, 0, 0, 0, 0 );

   // apply the terrains
   //LS::Create();
   //LS::flushCommands();

   // Except for this, this can stay :P
   exec( %world @ ".terrain.cs");

	appendSearchPath();
   //eval(Terrain:: @ %world @ "::createGridFile();");
   //eval(Terrain:: @ %world @ "::setRules();");
   //eval(Terrain:: @ $terrainTypes[%ti, type] @ "::create();");

	appendSearchPath();
   //LS::Textures( %dmlName, %gridName );
   //LS::parseCommands();
   //LS::ApplyLandscape();
   //LS::ApplyTextures();

   //saveTerrain( "terrain", %tedVol );

   reCalcCRC();

   newObject( tedVol, SimVolume, $folder @ $terrainName @ ".ted.vol");
   addToSet( "MissionGroup\\Volumes", "tedVol" );

   deleteObject(8);
   
   //Load the multiple terrains
   //The 3rd terrain is needed to correct a bug in the way vehicles move (particularly with tanks)
   newObject( "terrain3", "SimTerrain", "Load", $terrainName @ ".ted.dtf", $terrain1Xoffset, $terrain1Yoffset, $terrain3Zoffset, $terrain1Xrotation, $terrain1Yrotation, $terrain1Zrotation  );
   newObject( "terrain2", "SimTerrain", "Load", $terrainName @ ".ted.dtf", $terrain2Xoffset, $terrain2Yoffset, $terrain2Zoffset, $terrain2Xrotation, $terrain2Yrotation, $terrain2Zrotation  );
   newObject( "terrain",  "SimTerrain", "Load", $terrainName @ ".ted.dtf", $terrain1Xoffset, $terrain1Yoffset, $terrain1Zoffset, $terrain1Xrotation, $terrain1Yrotation, $terrain1Zrotation  );
      
   //Set detail levels on each terrain
   setTerrainDetail( "terrain3", 120.0, 140 );
   setTerrainVisibility( "terrain3", $visibleDistance, $hazeDistance, 1000, 2000 );
   setTerrainDetail( "terrain2", 120.0, 140 );
   setTerrainVisibility( "terrain2", $visibleDistance, $hazeDistance, 1000, 2000 );
   setTerrainDetail( "terrain", 120.0, 140 );
   setTerrainVisibility( "terrain",  $visibleDistance, $hazeDistance, 1000, 2000 );
   
   //Add them to the Missiongroup   
   addToSet( "MissionGroup\\World","terrainPal","terrain","terrain2","terrain3","sky","planet" );

   if ( $worldTypes[%wi, stars, %si] == "true" )
   {
      newObject( "stars", StarField );
      addToSet( "MissionGroup\\World", "stars" );
   }

   reCalcCRC();

   saveMission( $server::MissionFileName );

   //--------------------------------------------------------------

//   exec( $server::MissionFileName );
   onMissionPreLoad();
   preLoadExplosionTables();
   preLoad();
   resetGhostManager();

   serverResetScores();
   initScoreBoard();
   onMissionLoad();

   focusClient();

   GuiLoadContentCtrl("simCanvas", "play.gui");

   drop();
}

//-----------------------------------------------------------------------

function NewMission::exit(%mission)
{
   focusClient();

   // get rid of the dialog and do mouse things
   if(!$NM::cursorOn)
      cursorOff(simCanvas);
   
   // check if hit cancel
   if(%mission != "")
      schedule("NewMission::doCreate(\"" @ %mission @ "\");", 1); 
   else
      GuiPopDialog(simCanvas, 0);
}

//-----------------------------------------------------------------------
// this needs to be scheduled because the create is called through
// a buttons onaction (the button gets destroyed by this function)
function NewMission::doCreate(%mission)
{
   $ConnectedToServer = false;
   deleteServer();
//   purgeResources();
   deleteObject(ConsoleScheduler);
   newObject(ConsoleScheduler, SimConsoleScheduler);
   exec(editor);  // pre-load the editor
   createServer(%mission, false);
}

//-----------------------------------------------------------------------

function NewMission::reset()
{
   // initialize all the tables...
   exec(worlds);
   exec(terrains);

   // set the defaults 'NM for NewMission'
   if($NewMissionName != "")
   {
      $NM::missionName = $NewMissionName;
      $NewMissionName = "";
   }
   else
      $NM::missionName        = "NewMission";
   $NM::missionIndex       = 0;
   $NM::worldIndex         = 0;
   $NM::skyIndex           = 0;
   $NM::terrainIndex       = 0;
   $NM::terrainSeed        = 0;

   NewMission::setValues();

   // get the mouse setup
   $NM::cursorOn = isCursorOn(simCanvas);
   if(!$NM::cursorOn)
      cursorOn(simCanvas);
}

//-----------------------------------------------------------------------
// initialization

GuiPushDialog(simCanvas, "newmission.gui");
NewMission::reset();
   
// end initialization
//-----------------------------------------------------------------------

function NewMission::createServer()
{
   focusServer();
   deleteServer();
//   purgeResources();
   newServer();
   focusServer();
   initializeServer();

   // Create network delegate
   if ($server::UDPPortNumber == "")
   {
      $server::UDPPortNumber = 29001;
   }

   newObject( serverDelegate, ESCSDelegate, true, "IP", $server::UDPPortNumber);

   newObject( "", SimConsoleScheduler );
   newObject( "", ChatDispatcher );

   if (isObject(serverDelegate))
   {
      $Gui::ServerCreated = true;
   }

   # Set up for rendering...
   focusClient();
   cycleCamera();
   flushTextureCache( "simCanvas" );

   # Start a client
   connect("IP:127.0.0.1:29001");
}

